# SANS tags:
#       QUERY nid= vote_cutoff= LSEQ=
#       SBJCT VOTE= TUPS= PIDE= LALI= BITS= EVALUE= DIAG= LSEQ=
# computed:
#       qpid, spid
#       qcov,scov = LALI/LSEQ
#       desc, species, gene
#       rank
#       qseq, seq

import re,sys

class XMLParser:
    def __init__(self,queryspecies="auto"):
	    """	If queryspecies is "auto" then assume query comes from Uniprot and species is taken from OS= tag in header.
	    """
	    self.autospecies=(queryspecies=="auto") # Uniprot query
	    self.queryspecies=queryspecies
	    self.iquery=0
	    self.init_entry()
	    self.init_sbjct()
	    self.inquery=0
	    self.insbjct=0

    def stream(self,indata,header=False, bracket=False, output_sseq=False, output_ranges=False):
	self.init_entry()
	# header line
	result=""
	if bracket: result='#{\n'
	if header: 
		result += "\t".join('nid isquery qpid spid qcov scov bits pide lali desc species qseq vote genename'.split())
		if output_ranges: 
			result += "\t"
			result += "\t".join('qfrom qto qseqlen sfrom sto sseqlen'.split())
		result += "\n"
	if bracket: result += "#}\n"
	iline=0
	sseq="n.d." # default: no subjct sequence output
	while iline < len(indata):
		line=indata[iline]
		iline+=1
		if not line: continue
		if line[0]=='#': continue
		line=line.strip()
		if len(line)<=1: continue
		# FASTA part
		if line[0]=='>':
			try:
				pid,hdr=line[1:].split(" ",1)
			except:
				pid=line[1:]
				hdr=line[1:]
			# extract species from hdr OS=... GN=...
			if self.inquery==1 and self.autospecies:
				try:
					self.queryspecies=re.sub(r' \w{2}=.*$',"",hdr[hdr.index(r"OS=")+3:])
				except:
					self.queryspecies="unknown"
			if self.insbjct:
				try:
                                        genename=re.sub(r' \w{2}=.*$',"",hdr[hdr.index(r"GN=")+3:])
                                except:
                                        genename=""

				try: 
					species=re.sub(r' \w{2}=.*$',"",hdr[hdr.index(r"OS=")+3:])
				except:
					species="unknown"
    			# remove uniprot tags from desc OS=... GN=... PE=...
    			desc=re.sub(r' \w{2}=.*$',"",hdr)
			# sequence is next line from header line
			seq=indata[iline].rstrip()
			iline+=1
			if self.inquery==1:
				# print query row
				self.qpid=pid
				self.qseqlen=self.lseq
				genename=""
                                result += "\t".join([str(self.iquery),"1",self.qpid,self.qpid,"1.00","1.00","0.0","1.00",str(self.qseqlen),desc,self.queryspecies,seq,"0",genename]) 
				if output_ranges: 
					result += "\t"
					result += "\t".join(["1",str(self.qseqlen),str(self.qseqlen),"1",str(self.qseqlen),str(self.qseqlen)])
			elif self.insbjct==1:
				# compute qcov,scov
				qcov=str(self.lali/float(self.qseqlen))
				scov=str(self.lali/float(self.lseq))
				# print sbjct row
				if output_sseq: sseq=seq.rstrip()
				result += "\t".join([str(self.iquery),"0",self.qpid,pid,qcov,scov,self.bits,self.pide,str(self.lali),desc,species,sseq,self.vote,genename])
				if output_ranges:
					result += "\t" 
					result += "\t".join([str(self.qfrom),str(self.qto),str(self.qseqlen),str(self.sfrom),str(self.sto),str(self.lseq)])
			result += "\n"
		else: # parse XML tags
			line=re.sub(r'\z00','',line)
			x=line.split()
			i=0
			while i<len(x):
				if re.search('<QUERY',x[i]):
					self.init_entry()
					self.inquery=1
					self.insbjct=0
					self.iquery+=1
					if bracket: result += "#{\n"
				elif x[i]=='<SBJCT':
					self.init_sbjct()
					self.inquery=0
					self.insbjct=1
				elif x[i]=='LALI=':
					self.lali=int(x[i+1])
				elif x[i]=='VOTE=':
					self.vote=str(x[i+1])
				elif x[i]=='PIDE=':
					self.pide=str(x[i+1])
				elif x[i]=='BITS=':
					self.bits=str(x[i+1])
				elif x[i]=='LSEQ=':
					self.lseq=int(x[i+1]) 
				elif x[i]=='QFROM=':
					self.qfrom=int(x[i+1])
                                elif x[i]=='QTO=':
                                        self.qto=int(x[i+1])
                                elif x[i]=='SFROM=':
                                        self.sfrom=int(x[i+1])
                                elif x[i]=='STO=':
                                        self.sto=int(x[i+1])
 				elif x[i]=='</SBJCT>':
					self.insbjct=0
				elif x[i]=='</QUERY>':
					self.inquery=0
					if bracket: result += "#}\n"
				elif x[i]=='nid=':
					self.nid=x[i+1] # this is not output, self.iquery is
				i+=1
	#result += "\n#}\n"
	return(result)

    def init_entry(self):
	self.qseqlen=1
	self.qpid=''

    def init_sbjct(self):
	self.lseq=1
	self.pid=""
	self.bits="0.0"
	self.pide="0.0"
	self.lali=0
	self.vote="0"

if __name__=="__main__":
	queryspecies=sys.argv[1]
	p=XMLParser(queryspecies)
	indata=sys.stdin.readlines()
	print p.stream(indata,output_ranges=True,output_sseq=True)

