#!/usr/bin/perl

# inputs: go_data = goid, parentlist; gene_goa from STDIN: accession_number, GO:goid
# outputs: godict.txt (STDOUT), GO.IEA.counts (STDERR)

use strict;

if($#ARGV<0) { die "USAGE: grep ^UniProtKB gene_association.goa_uniprot | cut -f 2,5 | perl $0 go_data > godict.txt 2> GO.IEA.counts\n"; }
my($GO_DATA)=@ARGV;

my %gocount;
my %ontology;
my %name;

# hash GO-tree
open(IN,"<$GO_DATA") || die "Can't open $GO_DATA\n";
my %plist;
while(<IN>) {
	chomp;
	my($goid, $namespace, $name, $parentlist)=split(/\t/);
	next if($namespace eq '');
	if($namespace eq 'biological_process') { $namespace='BP'; }
	elsif($namespace eq 'molecular_function') { $namespace='MF'; }
	else { $namespace='CC'; }
	$ontology{$goid}=$namespace;
	$name{$goid}=$name;
	$gocount{$goid}=0;
	$plist{$goid}=join(",",sort split(/,/,$parentlist));
}
close(IN);

# read acc, go from STDIN
my $oldacc="";
my %golist;
while(<STDIN>) {
	chomp;
	my($acc,$goid)=split(/\t/);
	$goid=~s/GO://;
	$gocount{$goid}++; # direct assignment
	#warn "# got $acc $goid $gocount{$goid} $plist{$goid}\n";
	if($acc ne $oldacc) {
		# print previous
		if($oldacc ne "") { 
			my(@x)=keys %golist;
			foreach (@x) { $gocount{$_}++; }
			print join("\t",$oldacc,join(",",@x)),"\n"; 
		}
		undef %golist;
		$oldacc=$acc;
	}
	$golist{$goid}=1;
	foreach my $p (split(/,/,$plist{$goid})) { 
		#if($p eq '0006636') { warn "# p = $p count = $gocount{$p}\n"; }
		$golist{$p}=1; 
	}
}
# print last
if($oldacc ne "") { 
        my(@x)=keys %golist;
        foreach (@x) { $gocount{$_}++; }
	print join("\t",$oldacc,join(",",sort @x)),"\n"; 
}

# output  gocounts to STDERR
foreach my $goid (keys %gocount) {
	warn join("\t",$gocount{$goid},$ontology{$goid},$goid,$name{$goid},$plist{$goid}),"\n";
}
